/*
 * can_api.c
 *
 *  Created on: Oct 22, 2020
 *      Author: nxa07657
 */

#include "Config.h"

/* include the following header file to use FlexCAN PD layer driver API and data type */
#include "flexcan_driver.h"
/* include the following header file to use FlexCAN hardware access layer driver API and data type */
#include "flexcan_hw_access.h"

/***********************************************************************************
 * Configure the following two initialization structures according the HW design.
 *  for this demo, PTE4 is configured as FlexCAN0 RXD pin
 ***********************************************************************************/
/* configure to work as CAN RXD function */
const pin_settings_config_t MUX_CAN0_RXD_PinConfig = {
	.base          = PORTE,
	.pinPortIdx    = 4u,
	.pullConfig    = PORT_INTERNAL_PULL_NOT_ENABLED,
	.passiveFilter = false,
	.driveSelect   = PORT_LOW_DRIVE_STRENGTH,
	.mux           = PORT_MUX_ALT5,
	.pinLock       = false,
	.intConfig     = PORT_DMA_INT_DISABLED,
	.clearIntFlag  = false,
	.gpioBase      = NULL,
	.digitalFilter = false,
};
/* GPIO input with interrupt pull-up enabled and IRQ falling edge interrupt enabled */
const pin_settings_config_t MUX_IRQ_PinConfig = {
	.base          = PORTE,
	.pinPortIdx    = 4u,
	.pullConfig    = PORT_INTERNAL_PULL_UP_ENABLED,
	.passiveFilter = false,
	.driveSelect   = PORT_LOW_DRIVE_STRENGTH,
	.mux           = PORT_MUX_AS_GPIO,
	.pinLock       = false,
	.intConfig     = PORT_INT_FALLING_EDGE,
	.clearIntFlag  = true,
	.gpioBase      = PTE,
	.direction     = GPIO_INPUT_DIRECTION,
	.digitalFilter = false,
};

bool can_wakeup_flag = false;  /* flag to indicate CAN wakeup event*/

uint32_t CAN_RX0_BufferIndex = 0;    /*buffer index for CAN RX MB0*/
uint32_t CAN_RX1_BufferIndex = 0;    /*buffer index for CAN RX MB1*/
uint32_t CAN_RX2_BufferIndex = 0;    /*buffer index for CAN RX MB2*/
uint32_t CAN_RxDataBufferIndex = 0; /*the latest CAN message data buffer index */

bool CAN_RX0_Completed_Flag = false; /* CAN RX MB0 flag used for sync between CAN TXRX ISR and application*/
bool CAN_RX1_Completed_Flag = false; /* CAN RX MB1 flag used for sync between CAN TXRX ISR and application*/
bool CAN_RX2_Completed_Flag = false; /* CAN RX MB2 flag used for sync between CAN TXRX ISR and application*/

can_message_t CAN_RX0_Message_Buffer[CAN_RX_Buffer_MAX];/*circle buffer for CAN RX MB0*/
can_message_t CAN_RX1_Message_Buffer[CAN_RX_Buffer_MAX];/*circle buffer for CAN RX MB1*/
can_message_t CAN_RX2_Message_Buffer[CAN_RX_Buffer_MAX];/*circle buffer for CAN RX MB2*/

bool BusOff_Flag = false; /* the bus off flag */

/* CAN hardware MB(Message Buffer) configuration for TX MB */
can_buff_config_t CAN_TX_MB_Config[] = {
		/* CAN hardware MB(Message Buffer) configuration for TX MB0 */
	{	.enableFD = true,
		.enableBRS = true,
		.fdPadding = 0xCC,
		.idType = CAN_MSG_ID_STD,
		.isRemote = false
	},
	/* CAN hardware MB(Message Buffer) configuration for TX MB1 */
	{
	  .enableFD = true,
	  .enableBRS = false,
	  .fdPadding = 0xCC,
	  .idType = CAN_MSG_ID_STD,
	  .isRemote = false
	 },
	 /* CAN hardware MB(Message Buffer) configuration for TX MB2 */
	 {
	   .enableFD = false,
	   .enableBRS = false,
	   .fdPadding = 0xCC,
	   .idType = CAN_MSG_ID_STD,
	   .isRemote = false
	  },
	  /* CAN hardware MB(Message Buffer) configuration for TX MB3 */
	  {
	    .enableFD = true,
	    .enableBRS = true,
	    .fdPadding = 0xCC,
	    .idType = CAN_MSG_ID_STD,
	    .isRemote = false
	  }
};

 /* CAN hardware MB(Message Buffer) configuration for RX MB0 */
can_buff_config_t CAN_RX_MB0_Config = {
  .enableFD = true,
  .enableBRS = true,
  .fdPadding = 0xCC,
  .idType = CAN_MSG_ID_EXT,
  .isRemote = false
 };
/* CAN hardware MB(Message Buffer) configuration for RX MB1 */
can_buff_config_t CAN_RX_MB1_Config = {
  .enableFD = false,
  .enableBRS = false,
  .fdPadding = 0xCC,
  .idType = CAN_MSG_ID_STD,
  .isRemote = false
 };

/* CAN hardware MB(Message Buffer) configuration for RX MB2 */
can_buff_config_t CAN_RX_MB2_Config = {
  .enableFD = false,
  .enableBRS = false,
  .fdPadding = 0xCC,
  .idType = CAN_MSG_ID_STD,
  .isRemote = false
 };

/* CAN message transmit buffer*/
can_message_t CAN_TX_Buffer ={
		.id = 0x100,
		.data = {0},
		.length = 64
};

/* Table of base addresses for CAN instances. */
static CAN_Type * const g_flexcanBasePtr[] = CAN_BASE_PTRS;

/*******************************************************************
 * The CAN TX and RX completed interrupt ISR callback
 *  1) use the new buffer to continue CAN message receive
 *  2) set the RX_Completed_Flag for application handle
 *
 *******************************************************************/
void CAN_TX_RX_Callback(uint32_t instance, can_event_t eventType, uint32_t objIdx, void *driverState)
{
	/*check and handle the TX complete event*/
	if(CAN_EVENT_TX_COMPLETE==eventType)
	{
		/*add your TX complete event handle codes here if needed*/

	}

	/*check and handle the RX complete event*/
	if(CAN_EVENT_RX_COMPLETE==eventType)
	{
		switch (objIdx)
		{
			case RX_MB0:
				{
					CAN_RX0_BufferIndex++; /*increase the RX buffer index*/

					/*use next circle buffer for the new CAN message receive*/
					CAN_Receive(&can_pal1_instance,objIdx,&CAN_RX0_Message_Buffer[CAN_RX0_BufferIndex%CAN_RX_Buffer_MAX]);

					CAN_RX0_Completed_Flag = true;/*set the RX completed flag*/

					break;
				}
			case RX_MB1:
				{
					CAN_RX1_BufferIndex++; /*increase the RX buffer index*/

					/*use next circle buffer for the new CAN message receive*/
					CAN_Receive(&can_pal1_instance,objIdx,&CAN_RX1_Message_Buffer[CAN_RX1_BufferIndex%CAN_RX_Buffer_MAX]);

					CAN_RX1_Completed_Flag = true;/*set the RX completed flag*/

					break;
				}
			case RX_MB2:
				{
					CAN_RX2_BufferIndex++; /*increase the RX buffer index*/

					/*use next circle buffer for the new CAN message receive*/
					CAN_Receive(&can_pal1_instance,objIdx,&CAN_RX2_Message_Buffer[CAN_RX2_BufferIndex%CAN_RX_Buffer_MAX]);

					CAN_RX2_Completed_Flag = true;/*set the RX completed flag*/

					break;
				}
			default: break;
		}
	}
}
/*************************************************************************************
 * the CAN error ISR callback
 * user can add handle code for :
 *
 * FlexCAN Error, Error Fast, Tx Warning, Rx Warning and Bus Off as well as Bus Off Done event
 *
 * Note: Here only add the bus-off recovery code for demonstration
 *************************************************************************************/
void CAN_Error_Callback(uint8_t instance, flexcan_event_type_t eventType, flexcan_state_t *flexcanState)
{
	/* temp variable to store ESR1 register value */
	uint32_t temp_ESR1_RegVal = 0;

	DEV_ASSERT(instance < CAN_INSTANCE_COUNT);

	CAN_Type * base = g_flexcanBasePtr[instance];

	if(FLEXCAN_EVENT_ERROR==eventType)
	{
		/*get the error flags for further process */
		temp_ESR1_RegVal = FLEXCAN_DRV_GetErrorStatus(instance);


		if(CAN_ESR1_BOFFINT_MASK&temp_ESR1_RegVal)
		{
			/* record the bus off event */
			BusOff_Flag = true;

#ifdef CAN_BUSOFF_RECOVERY_MANUAL
			/* abort current MB transmit */
			CAN_AbortTransfer(&can_pal1_instance,TX_MB0);
			CAN_AbortTransfer(&can_pal1_instance,TX_MB1);
			CAN_AbortTransfer(&can_pal1_instance,TX_MB2);
#endif  /* end of CAN_BUSOFF_RECOVERY_MANUAL */

			/* use can add their network management handle codes here  */
			PINS_DRV_ClearPins(RGB_LED_GPIO,(1<<RGB_GREEN_LED_PIN)); /*turn ON green RGB LED*/

			/*get the error flags again in case bus-off recovery have done */
			temp_ESR1_RegVal = FLEXCAN_DRV_GetErrorStatus(instance);
		}

		if(CAN_ESR1_BOFFDONEINT_MASK&temp_ESR1_RegVal)
		{
#ifdef CAN_BUSOFF_RECOVERY_MANUAL
			/* the bus off recovery condition(the Tx Error Counter (TXERRCNT) has finished counting 128 occurrences of 11 consecutive recessive bits on the CAN bus and is ready to leave Bus Off) has been detected*/
			/*
			 * clean BOFFREC bit to trigger bus off recovery
			 */
			FLEXCAN_BOFFREC_Config(instance, false);
#endif  /* end of CAN_BUSOFF_RECOVERY_MANUAL */

			/* wait for the FlexCAN has actually exited bus-off  */
  			while((base->ESR1&CAN_ESR1_FLTCONF_MASK)>=CAN_ESR1_FLTCONF(0x2));

			PINS_DRV_SetPins(RGB_LED_GPIO,(1<<RGB_GREEN_LED_PIN)); /*turn OFF green RGB LED*/

			BusOff_Flag = false; /* clean the flag */

#ifdef CAN_BUSOFF_RECOVERY_MANUAL
			/* disable automatic bus-off recovery again */
			FLEXCAN_BOFFREC_Config(instance, true);
#endif  /* end of CAN_BUSOFF_RECOVERY_MANUAL */

		}
	}
}
/*************************************************************
 * API to configure the FlexCAN bus off recovery method
 *
 * enable = true:  disable automatic recovering from Bus Off state
 * enable = false: enable automatic recovering from Bus Off state
 *                 it is also a MUST for user recovery
 *
 *************************************************************/
void FLEXCAN_BOFFREC_Config(uint32_t instance, bool enable)
{
	DEV_ASSERT(instance < CAN_INSTANCE_COUNT);

	CAN_Type * base = g_flexcanBasePtr[instance];

	FLEXCAN_EnterFreezeMode(base);

	if(true==enable)
	{
		/* set BOFFREC bit to disable automatic bus-off recovery */
		base->CTRL1 |= CAN_CTRL1_BOFFREC(enable);

		/* enable buss off done interrupt */
		base->CTRL2 |= CAN_CTRL2_BOFFDONEMSK(enable);
	}
	else
	{
		/* clean BOFFREC bit to enable automatic bus-off recovery */
		base->CTRL1 &= (uint32_t)(~CAN_CTRL1_BOFFREC_MASK);

		/* clean the bus-off done interrupt flag */
  		base->ESR1 = CAN_ESR1_BOFFDONEINT_MASK;
	}

	FLEXCAN_ExitFreezeMode(base);
}

/*************************************************************
 * API to require the FlexCAN to enter low-power mode
 *
 * mode = NORNAL_MODE:  enable the FlexCAN module
 * mode = STOP_MODE: disable the FlexCAN module
 *
 *************************************************************/
void FLEXCAN_Mode_Require(uint32_t instance, MODE_Type mode)
{
	DEV_ASSERT(instance < CAN_INSTANCE_COUNT);

	CAN_Type * base = g_flexcanBasePtr[instance];
	FLEXCAN_EnterFreezeMode(base);

	if(LOW_POER_MODE == mode)
	{
		FLEXCAN_Disable(base);
	}
	else
	{
		FLEXCAN_Enable(base);

		FLEXCAN_ExitFreezeMode(base);
	}
}

void Wakeup_IRQ_ISR(void)
{
	uint32_t port_int_flags =0;

	/* get the port interrupt flags */
	port_int_flags = PINS_DRV_GetPortIntFlag(MUX_IRQ_PinConfig.base);

	/* check whether the CAN_RXD IRQ happened */
	if(port_int_flags&(1<<MUX_IRQ_PinConfig.pinPortIdx))
	{
		can_wakeup_flag = true; /* set the flag */

		/* clear the interrupt flag */
		PINS_DRV_ClearPinIntFlagCmd(MUX_IRQ_PinConfig.base,MUX_IRQ_PinConfig.pinPortIdx);
	}
}

/*************************************************************
 * API to configure FlexCAN RXD wakeup enable/disable
 *
 * enable = true:  configure the RXD pin as GPIO input IRQ interrupt with
 *                 falling edge and install the wakeup ISR
 * enable = false: configure the pin to work as CAN RXD function with
 *                 IRQ interrupt disabled
 *
 *************************************************************/
void FlexCAN_RXD_Wakeup_Configure(uint32_t instance, bool enable)
{
	static uint8_t isr_installed = 0;

	DEV_ASSERT(instance < CAN_INSTANCE_COUNT);

	if(0==instance)
	{
		if(enable)
		{
			/* configure the RXD PIN as IRQ interrupt with input falling edge */
			PINS_DRV_Init(1, &MUX_IRQ_PinConfig);

			if(0 == isr_installed)
			{
				/* install the IRQ wakeup ISR */
				INT_SYS_InstallHandler(CAN_RXD_IRQn,Wakeup_IRQ_ISR, NULL);

				/* set the wakeup IRQ interrupt as the highest configurable priority */
				INT_SYS_SetPriority(CAN_RXD_IRQn, 0);

				/* set the flag to avoid repeat invoke */
				isr_installed = 1;
			}

			/* enable the IRQ interrupt */
			INT_SYS_EnableIRQ(CAN_RXD_IRQn);
		}
		else
		{
			/* configure the RXD PIN as IRQ interrupt with input falling edge */
			PINS_DRV_Init(1, &MUX_CAN0_RXD_PinConfig);

			/* disable the IRQ interrupt */
			INT_SYS_DisableIRQ(CAN_RXD_IRQn);
		}
	}
	/* please add RXD configurations for other FlexCAN modules as HW design*/
}

/*FUNCTION**********************************************************************
 *
 * Function Name : CAN_MB_CS_Update
 * Description   : This function update code and status configuration of
 *                 the specified FlexCAN module TX MB
 *  instance: CAN_PAL instance;
 *  buffIdx : TX MB index
 *  cs      : code and status input
 *
 * Implements    : CAN_MB_CS_Update_Activity
 *END**************************************************************************/
void CAN_MB_CS_Update(const can_instance_t * const instance, uint32_t buffIdx,uint32_t cs)
{
	buffIdx -= TX_MB0;  /* get the offset */

	/* get the CAN-FD enable configuration */
	CAN_TX_MB_Config[buffIdx].enableFD = (cs & CAN_MB_EDL_MASK)? true: false;

	/* get the bitrate switch enable configuration */
	CAN_TX_MB_Config[buffIdx].enableBRS = (cs & CAN_MB_BRS_MASK)? true: false;

	/* get the ID type configuration */
	CAN_TX_MB_Config[buffIdx].idType = (cs & CAN_CS_IDE_MASK)? CAN_MSG_ID_EXT: CAN_MSG_ID_STD;

	/* get the remote/data frame configuration */
	CAN_TX_MB_Config[buffIdx].isRemote = (cs & CAN_CS_RTR_MASK)?true: false;

	/* call CAN_PAL TX buffer configure API to update the CAN_PAL configuration */
	CAN_ConfigTxBuff(instance, (buffIdx + TX_MB0), &CAN_TX_MB_Config[buffIdx]);
}

/*FUNCTION**********************************************************************
 *
 * Function Name : CAN_User_Init
 * Description   : This function complete the CAN initialization
 *
 * Notes         : Please add the CAN initialization codes per requirements
 *
 * Implements    : CAN_User_Init_Activity
 *END**************************************************************************/
void CAN_User_Init(void)
{
	/* initialize the CAN module */
	CAN_Init(&can_pal1_instance, &can_pal1_Config0);

	/* install the TX/RX call back */
	CAN_InstallEventCallback(&can_pal1_instance,CAN_TX_RX_Callback, NULL);

	#ifdef CAN_BUSOFF_RECOVERY_MANUAL
	/* disable bus off automatic recovery */
	FLEXCAN_BOFFREC_Config(can_pal1_instance.instIdx, true);
	#endif

	/* install the FlexCAN error ISR callback, which will also enable error, TX warning, RX warning and bus-off interrupt */
	FLEXCAN_DRV_InstallErrorCallback(can_pal1_instance.instIdx, CAN_Error_Callback,NULL);

	/* configure the transmit MB0 */
	CAN_ConfigTxBuff(&can_pal1_instance,TX_MB0, &CAN_TX_MB_Config[0]);
	/* configure the transmit MB1 */
	CAN_ConfigTxBuff(&can_pal1_instance,TX_MB1, &CAN_TX_MB_Config[1]);
	/* configure the transmit MB2 */
	CAN_ConfigTxBuff(&can_pal1_instance,TX_MB2, &CAN_TX_MB_Config[2]);
	/* configure the transmit MB3 */
	CAN_ConfigTxBuff(&can_pal1_instance,TX_MB3, &CAN_TX_MB_Config[3]);

	/* configure the receive MB0 with ACK ID for ID filter */
	CAN_ConfigRxBuff(&can_pal1_instance,RX_MB0, &CAN_RX_MB0_Config, RX_ACK_ID0);
	/* configure the RX MB1 ID filter, use individual with 4 LSB ID masked */
	CAN_SetRxFilter(&can_pal1_instance,CAN_MSG_ID_EXT,RX_MB0, RX_ACK_ID0_MASK);

	/* configure the receive MB1 with ACK ID for ID filter */
	CAN_ConfigRxBuff(&can_pal1_instance,RX_MB1, &CAN_RX_MB1_Config, RX_ACK_ID1);
	/* configure the RX MB1 ID filter, use individual with 4 LSB ID masked */
	CAN_SetRxFilter(&can_pal1_instance,CAN_MSG_ID_STD,RX_MB1, RX_ACK_ID1_MASK);

	/* configure the receive MB2 with ACK ID for ID filter */
	CAN_ConfigRxBuff(&can_pal1_instance,RX_MB2, &CAN_RX_MB2_Config, RX_ACK_ID2);
	/* configure the RX MB2 ID filter, use individual with ID8~ID9 ID masked */
	CAN_SetRxFilter(&can_pal1_instance,CAN_MSG_ID_STD,RX_MB2, RX_ACK_ID2_MASK);
}
